@@ -1,13 +1,14 @@
 #ifndef FS_CEPH_IOCTL_H
 #define FS_CEPH_IOCTL_H
 
+#include "include/int_types.h"
+
 #if defined(__linux__)
 #include <linux/ioctl.h>
 #include <linux/types.h>
 #elif defined(__FreeBSD__)
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include "include/inttypes.h"
 #endif
 
 #define CEPH_IOCTL_MAGIC 0x97
