@@ -14,10 +14,14 @@
 # include <cryptopp/md5.h>
 # include <cryptopp/sha.h>
 # include <cryptopp/hmac.h>
+
+// reinclude our assert to clobber the system one
+# include "include/assert.h"
+
 namespace ceph {
   namespace crypto {
     void assert_init();
-    void init();
+    void init(CephContext *cct);
     void shutdown();
 
     using CryptoPP::Weak::MD5;
@@ -44,7 +48,7 @@ namespace ceph {
 // the crypto operations. We don't want to contaminate too much code
 // with error checking, and just say these really should never fail.
 // This assert MUST NOT be compiled out, even on non-debug builds.
-# include "assert.h"
+# include "include/assert.h"
 
 // ugly bit of CryptoPP that we have to emulate here :(
 typedef unsigned char byte;
@@ -52,7 +56,7 @@ typedef unsigned char byte;
 namespace ceph {
   namespace crypto {
     void assert_init();
-    void init();
+    void init(CephContext *cct);
     void shutdown();
     class Digest {
     private:
