@@ -24,7 +24,7 @@ public:
   map<string,pool_stat_t> pool_stats;
 
   MGetPoolStatsReply() : PaxosServiceMessage(MSG_GETPOOLSTATSREPLY, 0) {}
-  MGetPoolStatsReply(uuid_d& f, tid_t t, version_t v) :
+  MGetPoolStatsReply(uuid_d& f, ceph_tid_t t, version_t v) :
     PaxosServiceMessage(MSG_GETPOOLSTATSREPLY, v),
     fsid(f) {
     set_tid(t);
@@ -42,7 +42,7 @@ public:
   void encode_payload(uint64_t features) {
     paxos_encode();
     ::encode(fsid, payload);
-    ::encode(pool_stats, payload);
+    ::encode(pool_stats, payload, features);
   }
   void decode_payload() {
     bufferlist::iterator p = payload.begin();
